
#ifndef lint
static const char revid[] = 
	"@(#)$Id: LoadRDFCommand.cpp,v 1.1 2002/04/10 21:42:16 neves Exp $";
#endif // not lint

#if defined(__CYGWIN__)	// Required to compile under cygwin
#  define XMLPARSEAPI
#endif

#include "LoadRDFCommand.hh"

#include <cstdlib>
#include <fstream>
#include <iostream>
#include <sstream>

#ifdef DAMLDB_OUTLINE
#define inline
#include "LoadRDFCommand.icc"
#undef inline
#endif

#ifdef CCXX_NAMESPACES
namespace damldb {
#endif

  // CONSTANTS
  const int	    kBUFSIZ		= 1024;

  ////////////////////////////////////////////////////////////

  void 
  LoadRDFCommand::info(const string& p, const string& p2) const
  {
    if (verbose_)
      {
	cerr << "INFO: " << p << ' ' << p2 << endl;
      }
  }

  ////////////////////////////////////////////////////////////

  void 
  LoadRDFCommand::warning(const string& p, const string& p2) const
  {
    if (verbose_)
      {
	cerr << "WARN: " << p << ' ' << p2 << endl;
      }
  }

  ////////////////////////////////////////////////////////////

  void
  LoadRDFCommand::error(const string& p, const string& p2) const
  {
    cerr << "ERROR: " << p << ' ' << p2 << endl;
    exit(EXIT_FAILURE);
  }

  ////////////////////////////////////////////////////////////

  void
  LoadRDFCommand::statement_handler(void *user_data,
				    RDF_SubjectType subject_type,
				    const XML_Char* subject,
				    const XML_Char* predicate,
				    int             ordinal,
				    RDF_ObjectType  object_type,
				    const XML_Char* object,
				    const XML_Char* xml_lang )
  {
    // local statistics
    stmt_count_++;

    // sanity checks
    if (subject == NULL)
      {
	warning("RDF parser - NULL subject");
	return;
      }
    if (predicate == NULL)
      {
	warning("RDF parser - NULL predicate");
	return;
      }
    if (object == NULL)
      {
	warning("RDF parser - NULL object");
	return;
      }

    Statement statement =
      db_->add_statement(uri_.c_str(), subject, predicate, object);

    // mark literals
    switch (object_type)
      {
      case RDF_OBJECT_TYPE_RESOURCE:
	break;
      case RDF_OBJECT_TYPE_LITERAL:
      case RDF_OBJECT_TYPE_XML:
	statement.flags |= STATEMENT_FLAG_LITERAL_OBJECT;
	break;
      default:
	{
	  ostringstream ost;
	  ost << '(' << object_type << ')';
	  warning("RDF parser - unexpected object_type", ost.str().c_str());
	}
      }
  }

  ////////////////////////////////////////////////////////////

  extern "C"
  void c_statement_handler(void *user_data,
			   RDF_SubjectType subject_type,
			   const XML_Char* subject,
			   const XML_Char* predicate,
			   int             ordinal,
			   RDF_ObjectType  object_type,
			   const XML_Char* object,
			   const XML_Char* xml_lang )
  {
    LoadRDFCommand* command = reinterpret_cast<LoadRDFCommand*>(user_data);
    command->statement_handler(user_data,
			       subject_type,
			       subject,
			       predicate,
			       ordinal,
			       object_type,
			       object,
			       xml_lang);
  }


  ////////////////////////////////////////////////////////////

  void
  LoadRDFCommand::load()
  {
    ifstream is(path_.c_str());

    if (!is.is_open())
      { error(path_); }

    RDF_Parser rdf_parser = RDF_ParserCreate(NULL);


    RDF_SetUserData(rdf_parser, this);
    RDF_SetStatementHandler(rdf_parser, c_statement_handler);
    RDF_SetBase(rdf_parser, uri_.c_str());

    char buf[kBUFSIZ];
    bool done = false;

    do
      {
	is.read(buf, sizeof(buf));

	size_t len = is.gcount();
	done = (len < sizeof(buf));

	if ( ! RDF_Parse(rdf_parser, buf, len, done ) )
	  {
	    cerr << "**** ERROR **** : " << uri_ << " \n\t";
	    cerr << XML_ErrorString( XML_GetErrorCode( RDF_GetXmlParser( rdf_parser ) ) );
	    cerr << " at line ";
	    cerr << XML_GetCurrentLineNumber( RDF_GetXmlParser( rdf_parser ) ) << endl;

	    done = true;;
	  }
      }
    while( ! done );

    db_->sync();
    is.close();

    ostringstream ost;
    ost << stmt_count_ << " " << uri_;
    info(ost.str());
  }

#ifdef CCXX_NAMESPACES
} // namespace damldb
#endif
