
#ifndef lint
static const char revid[] = 
	"@(#)$Id: rdf2damldb.cpp,v 1.2 2002/04/10 23:45:12 neves Exp $";
#endif // not lint

#include <damldb/DAMLDB.hh>
#include <damldb/IDamlDB.hh>
#include <damldb/Timer.hh>

#include "LoadRDFCommand.hh"

#include <cstdlib>
#include <iostream>
#include <stdexcept>
#include <vector>

#ifdef CCXX_NAMESPACES
namespace damldb {
USES(std,string)
#endif

  // CONSTANTS

  const int	    kOPEN_MODE		= iodb::in | iodb::out | iodb::nocreate;
  const string	    kHELP_FLAG		= "--help";
  const string	    kLOAD_FLAG		= "--load";
  const string	    kTIME_FLAG		= "--time";

  // TYPES

  typedef vector<LoadRDFCommand> CommandList;

  ////////////////////////////////////////////////////////////

  static
  bool
  check_opt_flag(const string& arg,
		 const string& flag,
		 const bool acceptShortFlag)
  {
    return ((acceptShortFlag
	     && arg.size() == 2
	     && arg == flag.substr(1,2))
	    || arg == flag);
  }

  ////////////////////////////////////////////////////////////

  static
  void
  usage(const int argc, const char *const *argv)
  {
    cerr << "Usage:  " << argv[0] << " [--time] {--load file uri} ..." << endl;
    exit(EXIT_FAILURE);
  }

#ifdef CCXX_NAMESPACES
} //namespace damldb
#endif

////////////////////////////////////////////////////////////

#ifdef CCXX_NAMESPACES

/* FIXME: USING(damldb) Generates an INTERNAL COMPILER ERROR under MS VC98 -- P.N. */

#  if !defined(_MSC_VER)
	USING(damldb);
#  else
	USES(damldb,CommandList)
	USES(damldb,DAMLDB)
	USES(damldb,IDamlDB)
	USES(damldb,LoadRDFCommand)
	USES(damldb,Timer)
	USES(damldb,iodb)
	USES(damldb,kHELP_FLAG)
	USES(damldb,kLOAD_FLAG)
	USES(damldb,kOPEN_MODE)
	USES(damldb,kTIME_FLAG)
#  endif /* !_MSC_VER */

#endif /* CCXX_NAMESPACES */


int
main(int argc, char **argv)
{
  CommandList commands;
  bool        showTime    = false;
  int         exit_status = EXIT_SUCCESS;

  try
    {
      { // FIXME: MS VC98 for-loop scoping is broken. Add a new scope
	for (int i = 1; i < argc; i++)
	  {
	    char *arg = argv[i];
	    if (check_opt_flag(string(arg), kTIME_FLAG, true))
	      {
		showTime = true;
	      }
	    else if (check_opt_flag(string(arg), kHELP_FLAG, true))
	      {
		usage(argc, argv);
	      }
	    else if (check_opt_flag(string(arg), kLOAD_FLAG, true))
	      {
		if (i + 2 >= argc)
		  {
		    THROW(invalid_argument("--load flag requires two arguments"));
		  }

		LoadRDFCommand aCommand(argv[i+1], argv[i+2]);
		commands.push_back(aCommand);

		i += 2;
	      }
	    else
	      {
		usage(argc, argv);
	      }
	  }
      } // FIXME END: MS VC98 for-loop scoping workaround

      // check for something to do
      if (distance(commands.begin(), commands.end()) == 0)
	{
	  usage(argc, argv);
	}

      DAMLDB::set_open_mode(kOPEN_MODE);
      IDamlDB& db = *DAMLDB::getInstance()->db;

      CommandList::iterator i;
      Timer timer;

      for (i = commands.begin(); i != commands.end(); ++i)
	{
	      (*i).setDb(db);

	      if (showTime)
		{ timer.start(); }
	      
	      (*i).load();

	      if (showTime)
		{
		  timer.stop();

		  cout << "elapsed time = ";
		  cout << timer.elapsed_time().sec();
		  cout << ".";
		  cout << timer.elapsed_time().usec();
		  cout << " seconds ";
		  cout << endl;
		}
	}

    }
  catch( const exception& x)
    {
      cerr << "FATAL ERROR: " << x.what() << endl;
      exit_status = EXIT_FAILURE;
    }
  catch(...)
    {
      cerr << "Unknown exception ocurred" << endl;
      exit_status = EXIT_FAILURE;
    }

  return exit_status;
}
